@@ -1,10 +1,8 @@
-# $Id: FormTool.py,v 1.24.2.2 2003/05/15 15:36:02 dreamcatcher Exp $
-# $Source: /cvsroot/plone/CMFPlone/FormTool.py,v $
-__version__ = "$Revision: 1.24.2.2 $"[11:-2] + " " + "$Name:  $"[7:-2]
-
 from Products.Formulator.Form import FormValidationError, BasicForm
 from Products.Formulator import StandardFields
 from Products.CMFCore.utils import UniqueObject
+from Products.CMFFormController.ValidationError import ValidationError
+from Products.CMFFormController.ControllerState import ControllerState
 from Globals import InitializeClass
 from AccessControl import ClassSecurityInfo
 from OFS.SimpleItem import SimpleItem
@@ -20,33 +18,49 @@ from PloneUtilities import log_deprecated
 from interfaces.FormTool import IFormTool, ICMFForm
 from PloneUtilities import log as debug_log
 from NavigationTool import ScriptStatus
+from ZODB.POSException import ConflictError
+from Products.CMFPlone.PloneBaseTool import PloneBaseTool
 
 validator_cache = {}  # a place to stash cached validators
-# (we don't want to persist them in the ZODB since that would make debugging a big pain)
+
+# (we don't want to persist them in the ZODB since that would make
+# debugging a big pain)
 
 debug = 0  # enable/disable logging
 
-class FormTool(UniqueObject, SimpleItem):
+class FormTool(PloneBaseTool, UniqueObject, SimpleItem):
+    """ Form Tool """
     id = 'portal_form'
     meta_type= 'Plone Form Tool'
+    toolicon = 'skins/plone_images/error_icon.gif'
     security = ClassSecurityInfo()
-    __implements__ = IFormTool,
+    
+    __implements__ = (PloneBaseTool.__implements__, IFormTool, 
+                      SimpleItem.__implements__, )
 
     # special-purpose items used in the REQUEST
-    error_key = 'errors'                    # a dictionary used for storing form validation errors
-    form_submitted_key = 'form_submitted'   # supplies the name of the form submitted
-    id_key = 'id'                           # the id for the object being operated on (can be None)
+    # a dictionary used for storing form validation errors
+    error_key = 'errors'
+    # supplies the name of the form submitted
+    form_submitted_key = 'form_submitted'
+    # the id for the object being operated on (can be None)
+    id_key = 'id'
+
     validation_status_key = 'validation_status'
 
 
     def __before_publishing_traverse__(self, other, REQUEST):
-        # kill off everything in the traversal stack after the item after portal_form
-        stack = REQUEST.get('TraversalRequestNameStack', None)
-#        self.log('stack = ' + str(stack))
+
+        # kill off everything in the traversal stack after the item
+        # after portal_form
+
+        stack = REQUEST.get('TraversalRequestNameStack', [])
+        # self.log('stack = ' + str(stack))
         # get rid of extra portal_forms
-        while 'portal_form' in stack:
-            stack.remove('portal_form')
-        REQUEST.set('TraversalRequestNameStack', [stack[-1]])
+        if stack:
+            while 'portal_form' in stack:
+                stack.remove('portal_form')
+            REQUEST.set('TraversalRequestNameStack', [stack[-1]])
 
 
     def setValidators(self, form, validators=None):
@@ -78,18 +92,12 @@ class FormTool(UniqueObject, SimpleItem):
         return validators.split(',')
 
 
-    # expose ObjectManager's bad_id test to skin scripts
-    def good_id(self, id):
-        m = bad_id(id)
-        if m is not None:
-            return 0
-        return 1
-
-
     def createForm(self):
         """Returns a CMFForm object"""
-        # CMFForm wraps Formulator.BasicForm and provides some convenience methods that
-        # make BasicForms easier to work with from external methods.
+        # CMFForm wraps Formulator.BasicForm and provides some
+        # convenience methods that make BasicForms easier to work with
+        # from external methods.
+
         form = CMFForm()
         return form.__of__(self)
 
@@ -107,14 +115,19 @@ class FormTool(UniqueObject, SimpleItem):
 
 
     def __bobo_traverse__(self, REQUEST, name):
-#        self.log('in bobo_traverse (%s)' % name)
-        # We intercept traversal when a form needs validation.  We insert a FormValidator object
-        # into the traversal stack which, when published, does the proper validation and then
-        # hands off to the navigation tool to determine the correct object to publish.
+        # self.log('in bobo_traverse (%s)' % name)
+        # We intercept traversal when a form needs validation.  We
+        # insert a FormValidator object into the traversal stack
+        # which, when published, does the proper validation and then
+        # hands off to the navigation tool to determine the correct
+        # object to publish.
 
         # see if we are handling validation for this form
         validators = self.getValidators(name)
-#        self.log('validators = ' + str(validators))
+
+        REQUEST = self.REQUEST
+
+        # self.log('validators = ' + str(validators))
         if validators is None:
 
             # make sure this is a normal REQUEST
@@ -128,7 +141,7 @@ class FormTool(UniqueObject, SimpleItem):
                     return getattr(parent, name)
                 try:
                     return parent[name]
-                except:
+                except (KeyError, AttributeError):
                     pass
                 return REQUEST.RESPONSE.notFoundError(name)
             # no, this is a fake request issued by unrestrictedTraverse
@@ -137,31 +150,40 @@ class FormTool(UniqueObject, SimpleItem):
                     return getattr(self, name)
                 try:
                     return self[name]
-                except:
+                except (KeyError, AttributeError):
                     pass
                 raise KeyError, name
 
 
         # There are three potential points of entry to a form:
-        # 1) The form is accessed directly from a URL, e.g. http://plone/portal_form/link_edit:
-        #       In this case we have REQUEST.errors = None, REQUEST.form_submitted = None
-        # 2) The form is filled out and submitted.
-        #       In this case we have REQUEST.errors = None, REQUEST.form_submitted = 1
-        # 3) A form validation error occurs, and the form is invoked by the navigation tool
-        #       In this case we have REQUEST.errors != None, REQUEST.form_submitted = 1
+
+        # 1) The form is accessed directly from a URL,
+        #       e.g. http://plone/portal_form/link_edit: In this case
+        #       we have REQUEST.errors = None, REQUEST.form_submitted
+        #       = None
+
+        # 2) The form is filled out and submitted.  In this case we
+        # have REQUEST.errors = None, REQUEST.form_submitted = 1
+
+        # 3) A form validation error occurs, and the form is invoked
+        # by the navigation tool In this case we have REQUEST.errors
+        # != None, REQUEST.form_submitted = 1
         #
         # We only need to invoke the validators for case (2)
 
+
         # see if we need to invoke validation
         errors = REQUEST.get(self.error_key, None)
-        # Make sure errors is something generated by us, not something submitted via POST or GET
+        # Make sure errors is something generated by us, not something
+        # submitted via POST or GET
+
         if not type(errors) != type({}):
             errors = None
         form_submitted = REQUEST.get(FormTool.form_submitted_key, None)
 
-        # We wrap the object in the acquisition layer of the parent of the FormTool
-        # so that subsequent forms will operate on the FormTool's context and not
-        # on the FormTool.
+        # We wrap the object in the acquisition layer of the parent of
+        # the FormTool so that subsequent forms will operate on the
+        # FormTool's context and not on the FormTool.
         do_validate = form_submitted and not errors
 
         if not do_validate:
@@ -172,8 +194,9 @@ class FormTool(UniqueObject, SimpleItem):
             else:
                 return REQUEST.RESPONSE.notFoundError("%s\n" % (name))
 
-#        self.log('returning validator')
-        return FormValidator(name, validators, aq_parent(self)).__of__(aq_parent(self)) # wrap in acquisition layer
+            # self.log('returning validator')
+        validator = FormValidator(name, validators, aq_parent(self))
+        return validator.__of__(aq_parent(self)) # wrap in acquisition layer
 
 
     # DEPRECATED
@@ -189,6 +212,14 @@ class FormTool(UniqueObject, SimpleItem):
         log_deprecated('getValidator has been marked for deprecation.  Please use getValidators instead.')
         return self.getValidators(form)[0]
 
+    # DEPRECATED
+    def good_id(self, id):
+        log_deprecated('good_id has been marked for deprecation.  Please use plone_utils.good_id instead.')
+        m = bad_id(id)
+        if m is not None:
+            return 0
+        return 1
+
 
     def log(self, msg, loc=None):
         """ """
@@ -204,9 +235,11 @@ InitializeClass(FormTool)
 
 class FormValidator(SimpleItem):
     """ """
-    # When a form needs validation, we publish a FormValidator object in place of the next form.
-    # The FormValidator calls a validator method on the variables in the REQUEST and then
-    # hands off to a page determined by the NavigationTool
+    # When a form needs validation, we publish a FormValidator object
+    # in place of the next form.  The FormValidator calls a validator
+    # method on the variables in the REQUEST and then hands off to a
+    # page determined by the NavigationTool
+
     security = ClassSecurityInfo()
 
     def __init__(self, form, validators, context):
@@ -214,7 +247,7 @@ class FormValidator(SimpleItem):
         self.validators = validators
         self.id = "Error" # This should only be seen if an error occurs
         self.title = None
-#        self.log("created validator")
+        # self.log("created validator")
 
     def __str__(self):
         return 'FormValidator, form=%s, validators=%s' % \
@@ -230,17 +263,23 @@ class FormValidator(SimpleItem):
             trace.append('Invoking validation')
             context = aq_parent(self)
             # invoke validation
-            (status, kwargs, trace) = self._validate(context, REQUEST, trace)
+
+            (status, kwargs, trace) = self._validate(context, self.REQUEST, trace)
 
             (obj, kwargs) = context.portal_navigation.getNextObject(context, self.form, status, trace, **kwargs)
             kwargs['REQUEST'] = self.REQUEST
-            # Set the FormValidator's id and title to those of the published object
-            # so that breadcrumbs can extract an id and title from the traversal stack.
+
+            # Set the FormValidator's id and title to those of the
+            # published object so that breadcrumbs can extract an id
+            # and title from the traversal stack.
+
             if obj:
                 self.id = getattr(obj, 'id', None)
                 self.title = getattr(obj, 'title', None)
             return apply(obj, (), kwargs)
 
+        except ConflictError:
+            raise
         except:
             nav = getToolByName(self, 'portal_navigation')
             nav.logTrace(trace)
@@ -256,29 +295,90 @@ class FormValidator(SimpleItem):
         """
         try:
             errors = {}
-            status = 'success'  # default return value if the validator list is empty
+            # default return value if the validator list is empty
+            status = 'success'
             kwargs = {}
             for validator in self.validators:
                 trace.append('Invoking %s' % validator)
-                v = getattr(context, validator, None)
+                v = context.restrictedTraverse(validator, default=None)
                 if v is None:
                     raise KeyError("Unable to find validator '%s' in context '%s'.  Check your skins path." % (validator, str(context)))
-                script_status = mapply(v, REQUEST.args, REQUEST,
-                                       call_object, 1, missing_name, dont_publish_class,
-                                       REQUEST, bind=1)
 
-                # The preferred return type for scripts will eventually be an object.
-                # Until then, preserve compatibility with 1.0 alpha 4
-                if type(script_status) == type(()):
-                    (status, errors, kwargs) = script_status
-                    kwargs['errors'] = errors
-                    script_status = ScriptStatus(status, kwargs, None)
-                    # disable deprecation warning for now
-#                    log_deprecated('Validator \'%s\' uses a return signature that has been marked for deprecation.  Validators should return a ScriptStatus object.' % validator)
+                # handle CMFFormController validators properly
+                if getattr(v, 'is_validator', 0):
+                    # get a controller_state object and populate it
+                    form_controller = getToolByName(context, 'portal_form_controller')
+                    try:
+                        controller_state = form_controller.getState(context, 1)
+                    except ValueError:
+                        controller_state = ControllerState()
+                        controller_state.set(context=context)
+                        controller_state.setButton(None)
+                        for k in REQUEST.keys():
+                            if k.startswith('form.button.'):
+                                controller_state.setButton(k[len('form.button.'):])
+                                break
+                    controller_state.setStatus(REQUEST.get('validation_status', 'success'))
+                    controller_state.setErrors(REQUEST.get('errors', {}))
+                    # XXX this is a bit of a hack -- lots of REQUEST keys could be
+                    # controller_state kwargs, but we don't know which ones.
+                    # portal_status_message is a common one.
+                    msg = REQUEST.get('portal_status_message', None)
+                    if msg:
+                        controller_state.setKwargs({'portal_status_message':msg})
+                    REQUEST.set('controller_state', controller_state)
+                    if controller_state.hasValidated(validator):
+                        continue
+                    try:
+                        controller_state = mapply(v, REQUEST.args, REQUEST,
+                                                  call_object, 1, missing_name, dont_publish_class,
+                                                  REQUEST, bind=1)
+                        controller_state._addValidator(validator)
+                    except ValidationError, e:
+                        # if a validator raises a ValidatorException, execution of
+                        # validators is halted and the controller_state is set to
+                        # the controller_state embedded in the exception
+                        controller_state = e.controller_state
+                        state_class = getattr(controller_state, '__class__', None)
+                        if state_class != ControllerState:
+                            raise Exception, 'Bad ValidationError state (type = %s)' % str(state_class)
+                        status = controller_state.getStatus()
+                        kwargs = controller_state.getKwargs()
+                        kwargs['errors'] = controller_state.getErrors()
+                        new_context = controller_state.getContext()
+                        break
+                    state_class = getattr(controller_state, '__class__', None)
+                    if state_class != ControllerState:
+                        raise Exception, 'Bad validator return type from validator %s (%s)' % (str(v), str(state_class))
 
-                status = script_status.status
-                kwargs = script_status.kwargs
-                new_context = script_status.new_context
+                    status = controller_state.getStatus()
+                    kwargs = controller_state.getKwargs()
+                    kwargs['errors'] = controller_state.getErrors()
+                    new_context = controller_state.getContext()
+
+                else:
+                    script_status = mapply(v, REQUEST.args, REQUEST,
+                                           call_object, 1, missing_name, dont_publish_class,
+                                           REQUEST, bind=1)
+
+                    # The preferred return type for scripts will
+                    # eventually be an object.  Until then, preserve
+                    # compatibility with 1.0 alpha 4
+
+                    if type(script_status) == type(()):
+                        (status, errors, kwargs) = script_status
+                        kwargs['errors'] = errors
+                        script_status = ScriptStatus(status, kwargs, None)
+
+                        # disable deprecation warning for now
+                        # log_deprecated('Validator \'%s\' uses a return
+                        # signature that has been marked for deprecation.
+                        # Validators should return a ScriptStatus object.'
+                        # % validator)
+
+                    status = script_status.status
+                    kwargs = script_status.kwargs
+                    new_context = script_status.new_context
 
                 self.REQUEST.set('validation_status', status)
                 for key in kwargs.keys():
@@ -289,6 +389,8 @@ class FormValidator(SimpleItem):
                     trace.append("\t context changed to '%s'" % str(context))
             trace.append('Validation returned (%s, %s)' % (status, str(kwargs)))
             return (status, kwargs, trace)
+        except ConflictError:
+            raise
         except:
             nav = getToolByName(self, 'portal_navigation')
             nav.logTrace(trace)
@@ -316,7 +418,7 @@ class CMFForm(BasicForm):
     __implements__ = ICMFForm,
 
     security.declareProtected('View', 'get_field')
-    def get_field(self, id):
+    def get_field(self, id, include_disabled=0):
         """Get a field of a certain id, wrapping in context of self
         """
         return self.fields[id].__of__(self)
@@ -393,24 +495,25 @@ class CMFForm(BasicForm):
                 # get rid of the old key
                 try:
                     del REQUEST[key]
-                except:
+                except (KeyError, AttributeError):
                     pass
                 # move the old value to 'field_' + key
                 # if there is already a value at 'field_' + key,
                 #    move it to 'field_field_' + key, and repeat
                 #    to prevent key collisions
-                newKey = 'field_' + key
+                newKey = 'field_%s' % key
                 newValue = REQUEST.get(newKey)
                 while newValue:
                     REQUEST[newKey] = value
-                    newKey = 'field_' + newKey
+                    newKey = 'field_%s' % newKey
                     value = newValue
                     newValue = REQUEST.get(newKey)
                 REQUEST[newKey] = value
 
         try:
-            result=self.validate_all(REQUEST)
+            result = self.validate_all(REQUEST)
         except FormValidationError, e:
+            result = {}
             for error in e.errors:
                 errors[error.field.get_value('title')]=error.error_text
 
@@ -419,13 +522,17 @@ class CMFForm(BasicForm):
             key = field.id
             value = 1
             while value:
-                key = 'field_' + key
-                value = REQUEST.get(key)
+                key = 'field_%s' % key
+                value = result.get(key[6:], None) or REQUEST.get(key)
                 if value:
                     REQUEST[key[6:]] = value
+                    try:
+                        del result[key[6:]]
+                    except (KeyError, AttributeError):
+                        pass
                     try:
                         del REQUEST[key]
-                    except:
+                    except (KeyError, AttributeError):
                         pass
 
         return errors
